package main

import "time"

type DBDate uint32

// YYYYMMDD max 8 digits.  int32.

func (d *DBDate) Year() int {
	return int(*d) / 10000
}

func (d *DBDate) Month() int {
	return (int(*d) / 100) % 100
}

func (d *DBDate) Day() int {
	return int(*d) % 100
}

// Set day - does no validation
func (d *DBDate) SetDay(day int) {
	var i int = int(*d)
	i -= (i % 100)
	i += day
	*d = DBDate(i)
}

// Make the first of the month
func (d *DBDate) First() {
	d.SetDay(1)
}

// Make end of month
func (d *DBDate) Last() {
	var value int = int(*d)
	var lastDay time.Time = time.Date(value/10000,
		time.Month((value/100)%100)+1,
		0, 2, 2, 2, 0, time.Local)
	*d = DBDate(lastDay.Year()*10000 +
		int(lastDay.Month())*100 + lastDay.Day())
}

func NewDBDate(year, month, day int) DBDate {
	return DBDate(year*10000 + month*100 + day)
}

func (d *DBDate) YMD() (int, int, int) {
	var i int = int(*d)
	return i / 10000, (i / 100) % 100, i % 100
}

// Convert time.Time to DBDate
func ToDBDate(point time.Time) DBDate {
	return DBDate(point.Year()*10000 +
		int(point.Month())*100 + point.Day())
}

// Convert DBDate to time.Time
func DBDateTime(point DBDate) time.Time {
	// I'm not sure about hours, minutes, seconds.

	var value int = int(point)
	var result time.Time = time.Date(value/10000,
		time.Month((value/100)%100),
		value%100, 2, 2, 2, 0, time.Local)
	return result
}

//
// Store dates as uint32
// YYYYMMDD
//
// So: Jan 21, 2022 => 20220121
//
